@@ -72,7 +72,7 @@ public:
    * @param allele_keys A vector containing thes allele keys to put in the MonolocusGenotype.
    * @return A MonolocusGenotype according to the number of alleles
    */
-  static std::auto_ptr<MonolocusGenotype> buildMonolocusGenotypeByAlleleKey(const std::vector<size_t> allele_keys) throw (Exception);
+  static std::unique_ptr<MonolocusGenotype> buildMonolocusGenotypeByAlleleKey(const std::vector<size_t> allele_keys) throw (Exception);
 };
 } // end of namespace bpp;
 
