@@ -41,6 +41,8 @@
 
 #include "PolymorphismSequenceContainer.h"
 
+#include <Bpp/Seq/SequenceTools.h>
+
 using namespace bpp;
 using namespace std;
 
@@ -62,6 +64,14 @@ PolymorphismSequenceContainer::PolymorphismSequenceContainer(size_t size, const
 
 /******************************************************************************/
 
+PolymorphismSequenceContainer::PolymorphismSequenceContainer(const vector<string>& names, const Alphabet* alpha) :
+  VectorSiteContainer(names, alpha),
+  ingroup_(names.size()),
+  count_(names.size()),
+  group_(names.size()) {}
+
+/******************************************************************************/
+
 PolymorphismSequenceContainer::PolymorphismSequenceContainer(const OrderedSequenceContainer& sc) :
   VectorSiteContainer(sc),
   ingroup_(sc.getNumberOfSequences(), true),
@@ -70,6 +80,36 @@ PolymorphismSequenceContainer::PolymorphismSequenceContainer(const OrderedSequen
 
 /******************************************************************************/
 
+PolymorphismSequenceContainer::PolymorphismSequenceContainer(const OrderedSequenceContainer& sc, bool count) :
+  VectorSiteContainer(sc.getAlphabet()),
+  ingroup_(),
+  count_(),
+  group_()
+{
+  if (sc.getNumberOfSequences() == 0) return; //done.
+
+  // Add first sequence:
+  addSequenceWithFrequency(sc.getSequence(0), 1);
+  for (size_t i = 1; i < sc.getNumberOfSequences(); ++i) {
+    const Sequence& seq = sc.getSequence(i);
+    //Check if this sequence already exists in this container:
+    bool exists = false;
+    for (size_t j = 0; !exists && j < getNumberOfSequences(); ++j) {
+      if (SequenceTools::areSequencesIdentical(getSequence(j), seq)) {
+        incrementSequenceCount(j); //We increase frequency, meaning that we discard this sequence name.
+        exists = true;
+      }
+    }
+    if (!exists) {
+      addSequenceWithFrequency(seq, 1);
+    }
+  }
+  ingroup_.resize(getNumberOfSequences(), true);
+  group_.resize(getNumberOfSequences());
+}
+
+/******************************************************************************/
+
 PolymorphismSequenceContainer::PolymorphismSequenceContainer(const SiteContainer& sc) :
   VectorSiteContainer(sc),
   ingroup_(sc.getNumberOfSequences(), true),
@@ -78,6 +118,36 @@ PolymorphismSequenceContainer::PolymorphismSequenceContainer(const SiteContainer
 
 /******************************************************************************/
 
+PolymorphismSequenceContainer::PolymorphismSequenceContainer(const SiteContainer& sc, bool count) :
+  VectorSiteContainer(sc.getAlphabet()),
+  ingroup_(),
+  count_(),
+  group_()
+{
+  if (sc.getNumberOfSequences() == 0) return; //done.
+  
+  // Add first sequence:
+  addSequenceWithFrequency(sc.getSequence(0), 1);
+  for (size_t i = 1; i < sc.getNumberOfSequences(); ++i) {
+    const Sequence& seq = sc.getSequence(i);
+    //Check if this sequence already exists in this container:
+    bool exists = false;
+    for (size_t j = 0; !exists && j < getNumberOfSequences(); ++j) {
+      if (SequenceTools::areSequencesIdentical(getSequence(j), seq)) {
+        incrementSequenceCount(j); //We increase frequency, meaning that we discard this sequence name.
+        exists = true;
+      }
+    }
+    if (!exists) {
+      addSequenceWithFrequency(seq, 1);
+    }
+  }
+  ingroup_.resize(getNumberOfSequences(), true);
+  group_.resize(getNumberOfSequences());
+}
+
+/******************************************************************************/
+
 PolymorphismSequenceContainer::PolymorphismSequenceContainer(const PolymorphismSequenceContainer& psc) :
   VectorSiteContainer(psc),
   ingroup_(psc.getNumberOfSequences()),
@@ -287,6 +357,16 @@ size_t PolymorphismSequenceContainer::getNumberOfGroups() const
 
 /******************************************************************************/
 
+bool PolymorphismSequenceContainer::hasOutgroup() const
+{
+  for (auto i: ingroup_) {
+    if (!i) return true;
+  }
+  return false;
+}
+
+/******************************************************************************/
+
 bool PolymorphismSequenceContainer::isIngroupMember(size_t index) const throw (IndexOutOfBoundsException)
 {
   if (index >= getNumberOfSequences())
@@ -462,3 +542,23 @@ unsigned int PolymorphismSequenceContainer::getSequenceCount(const std::string&
 
 /******************************************************************************/
 
+SiteContainer* PolymorphismSequenceContainer::toSiteContainer() const {
+  unique_ptr<VectorSiteContainer> sites(new VectorSiteContainer(getAlphabet()));
+  for (size_t i = 0; i < getNumberOfSequences(); ++i) {
+    const Sequence& seq = getSequence(i);
+    unsigned int freq = getSequenceCount(i);
+    if (freq > 1) {
+      for (unsigned int j = 0; j < freq; ++j) {
+        unique_ptr<Sequence> seqdup(seq.clone());
+        seqdup->setName(seq.getName() + "_" + TextTools::toString(j + 1));
+        sites->addSequence(*seqdup);
+      }
+    } else {
+      sites->addSequence(seq);
+    }
+  }
+  return sites.release();
+}
+ 
+/******************************************************************************/
+
