@@ -40,6 +40,8 @@
 
 #include "PolymorphismSequenceContainerTools.h"
 
+#include <Bpp/Seq/CodonSiteTools.h>
+
 using namespace bpp;
 using namespace std;
 
@@ -586,3 +588,54 @@ string PolymorphismSequenceContainerTools::getIngroupSpeciesName(const Polymorph
 }
 
 /******************************************************************************/
+
+PolymorphismSequenceContainer* PolymorphismSequenceContainerTools::getSynonymousSites(const PolymorphismSequenceContainer& psc, const GeneticCode& gCode)
+{
+  unique_ptr<PolymorphismSequenceContainer> psco(new PolymorphismSequenceContainer(psc.getSequencesNames(), psc.getAlphabet()));
+  for (size_t i = 0; i < psc.getNumberOfSites(); ++i) {
+    const Site& site = psc.getSite(i);
+    if (CodonSiteTools::isSynonymousPolymorphic(site, gCode)) {
+      psco->addSite(site);
+    }
+  }
+  for (size_t i = 0; i < psc.getNumberOfSequences(); i++)
+  {
+    psco->setSequenceCount(i, psc.getSequenceCount(i));
+    if (psc.isIngroupMember(i))
+      psco->setAsIngroupMember(i);
+    else
+    {
+      psco->setAsOutgroupMember(i);
+      psco->setGroupId(i, psc.getGroupId(i));
+    }
+  }
+  return psco.release();
+}
+
+/******************************************************************************/
+
+PolymorphismSequenceContainer* PolymorphismSequenceContainerTools::getNonSynonymousSites(const PolymorphismSequenceContainer& psc, const GeneticCode& gCode)
+{
+  unique_ptr<PolymorphismSequenceContainer> psco(new PolymorphismSequenceContainer(psc.getSequencesNames(), psc.getAlphabet()));
+  for (size_t i = 0; i < psc.getNumberOfSites(); ++i) {
+    const Site& site = psc.getSite(i);
+    if (!CodonSiteTools::isSynonymousPolymorphic(site, gCode)) {
+      psco->addSite(site);
+    }
+  }
+  for (size_t i = 0; i < psc.getNumberOfSequences(); i++)
+  {
+    psco->setSequenceCount(i, psc.getSequenceCount(i));
+    if (psc.isIngroupMember(i))
+      psco->setAsIngroupMember(i);
+    else
+    {
+      psco->setAsOutgroupMember(i);
+      psco->setGroupId(i, psc.getGroupId(i));
+    }
+  }
+  return psco.release();
+}
+
+/******************************************************************************/
+
