@@ -73,7 +73,7 @@ unsigned int SequenceStatistics::numberOfPolymorphicSites(const PolymorphismSequ
 {
   unsigned int s = 0;
   const Site* site = 0;
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -89,9 +89,31 @@ unsigned int SequenceStatistics::numberOfPolymorphicSites(const PolymorphismSequ
   return s;
 }
 
+double SequenceStatistics::frequencyOfPolymorphicSites(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown)
+{
+  double s = 0;
+  double n = 0;
+  const Site* site = 0;
+  unique_ptr<ConstSiteIterator> si;
+  if (gapflag)
+    si.reset(new CompleteSiteContainerIterator(psc));
+  else
+    si.reset(new SimpleSiteContainerIterator(psc));
+  while (si->hasMoreSites())
+  {
+    site = si->nextSite();
+    n++;
+    if (!SiteTools::isConstant(*site, ignoreUnknown))
+    {
+      s++;
+    }
+  }
+  return s / n;
+}
+
 unsigned int SequenceStatistics::numberOfParsimonyInformativeSites(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -111,7 +133,7 @@ unsigned int SequenceStatistics::numberOfParsimonyInformativeSites(const Polymor
 
 unsigned int SequenceStatistics::numberOfSingletons(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -128,7 +150,7 @@ unsigned int SequenceStatistics::numberOfSingletons(const PolymorphismSequenceCo
 
 unsigned int SequenceStatistics::numberOfTriplets(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -148,7 +170,7 @@ unsigned int SequenceStatistics::numberOfTriplets(const PolymorphismSequenceCont
 
 unsigned int SequenceStatistics::totalNumberOfMutations(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -165,15 +187,15 @@ unsigned int SequenceStatistics::totalNumberOfMutations(const PolymorphismSequen
 
 unsigned int SequenceStatistics::totalNumberOfMutationsOnExternalBranches(
   const PolymorphismSequenceContainer& ing,
-  const PolymorphismSequenceContainer& outg) throw (Exception)
+  const PolymorphismSequenceContainer& outg)
 {
   if (ing.getNumberOfSites() != outg.getNumberOfSites())
     throw Exception("ing and outg must have the same size");
   unsigned int nmuts = 0;
   const Site* site_in = 0;
   const Site* site_out = 0;
-  auto_ptr<ConstSiteIterator> si(new SimpleSiteContainerIterator(ing));
-  auto_ptr<ConstSiteIterator> so(new SimpleSiteContainerIterator(outg));
+  unique_ptr<ConstSiteIterator> si(new SimpleSiteContainerIterator(ing));
+  unique_ptr<ConstSiteIterator> so(new SimpleSiteContainerIterator(outg));
   while (si->hasMoreSites())
   {
     site_in = si->nextSite();
@@ -187,7 +209,7 @@ unsigned int SequenceStatistics::totalNumberOfMutationsOnExternalBranches(
 
 double SequenceStatistics::heterozygosity(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -204,7 +226,7 @@ double SequenceStatistics::heterozygosity(const PolymorphismSequenceContainer& p
 
 double SequenceStatistics::squaredHeterozygosity(const PolymorphismSequenceContainer& psc, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -239,7 +261,7 @@ std::vector<unsigned int> SequenceStatistics::gcPolymorphism(const PolymorphismS
   size_t nbSeq = psc.getNumberOfSequences();
   vector<unsigned int> vect(2);
   const Site* site = 0;
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -273,31 +295,37 @@ std::vector<unsigned int> SequenceStatistics::gcPolymorphism(const PolymorphismS
 // Diversity statistics
 // ******************************************************************************
 
-double SequenceStatistics::watterson75(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown)
+double SequenceStatistics::watterson75(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown, bool scaled)
 {
   double ThetaW;
   size_t n = psc.getNumberOfSequences();
-  unsigned int s = numberOfPolymorphicSites(psc, gapflag, ignoreUnknown);
   map<string, double> values = getUsefulValues_(n);
-  ThetaW = static_cast<double>(s) / values["a1"];
+  double s = 0;
+  if (scaled)
+    s = frequencyOfPolymorphicSites(psc, gapflag, ignoreUnknown);
+  else
+    s = static_cast<double>(numberOfPolymorphicSites(psc, gapflag, ignoreUnknown));
+  ThetaW = s / values["a1"];
   return ThetaW;
 }
 
-double SequenceStatistics::tajima83(const PolymorphismSequenceContainer& psc, bool gapflag)
+double SequenceStatistics::tajima83(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown, bool scaled)
 {
   size_t alphabet_size = psc.getAlphabet()->getSize();
   const Site* site = 0;
-  ConstSiteIterator* si = 0;
+  unique_ptr<ConstSiteIterator> si;
   double value2 = 0.;
+  double l = 0;
   if (gapflag)
-    si = new CompleteSiteContainerIterator(psc);
+    si.reset(new CompleteSiteContainerIterator(psc));
   else
-    si = new SimpleSiteContainerIterator(psc);
+    si.reset(new SimpleSiteContainerIterator(psc));
   while (si->hasMoreSites())
   {
     site = si->nextSite();
-    if (!SiteTools::isConstant(*site))
+    if (!SiteTools::isConstant(*site, ignoreUnknown))
     {
+      l++;
       double value = 0.;
       map<int, size_t> count;
       SymbolListTools::getCounts(*site, count);
@@ -320,8 +348,7 @@ double SequenceStatistics::tajima83(const PolymorphismSequenceContainer& psc, bo
       value2 += 1. - value;
     }
   }
-  delete si;
-  return value2;
+  return (scaled ? value2 / l : value2);
 }
 
 double SequenceStatistics::fayWu2000(const PolymorphismSequenceContainer& psc, const Sequence& ancestralSites)
@@ -378,7 +405,7 @@ unsigned int SequenceStatistics::dvk(const PolymorphismSequenceContainer& psc, b
    * This implementation uses unneeded SequenceContainer recopy and works on
    * string. It needs to be improved.
    */
-  auto_ptr<PolymorphismSequenceContainer> sc;
+  unique_ptr<PolymorphismSequenceContainer> sc;
   if (gapflag)
     sc.reset(PolymorphismSequenceContainerTools::getSitesWithoutGaps(psc));
   else
@@ -416,7 +443,7 @@ double SequenceStatistics::dvh(const PolymorphismSequenceContainer& psc, bool ga
    * This implementation uses unneeded SequenceContainer recopy and works on
    * string. It needs to be improved.
    */
-  auto_ptr<PolymorphismSequenceContainer> sc;
+  unique_ptr<PolymorphismSequenceContainer> sc;
   if (gapflag)
     sc.reset(PolymorphismSequenceContainerTools::getSitesWithoutGaps(psc));
   else
@@ -459,7 +486,7 @@ double SequenceStatistics::dvh(const PolymorphismSequenceContainer& psc, bool ga
 unsigned int SequenceStatistics::numberOfTransitions(const PolymorphismSequenceContainer& psc)
 {
   unsigned int nbT = 0;
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   const Site* site = 0;
   while (si->hasMoreSites())
   {
@@ -467,14 +494,13 @@ unsigned int SequenceStatistics::numberOfTransitions(const PolymorphismSequenceC
     // if (SiteTools::isConstant(*site) || SiteTools::isTriplet(*site)) continue;
     if (SiteTools::getNumberOfDistinctCharacters(*site) != 2)
       continue;
-    vector<int> seq = site->getContent();
-    int state1 = seq[0];
-    int state2 = seq[0];
-    for (size_t i = 1; i < seq.size(); i++)
+    int state1 = (*site)[0];
+    int state2 = (*site)[0];
+    for (size_t i = 1; i < site->size(); i++)
     {
-      if (state1 != seq[i])
+      if (state1 != (*site)[i])
       {
-        state2 = seq[i];
+        state2 = (*site)[i];
         break;
       }
     }
@@ -490,7 +516,7 @@ unsigned int SequenceStatistics::numberOfTransitions(const PolymorphismSequenceC
 unsigned int SequenceStatistics::numberOfTransversions(const PolymorphismSequenceContainer& psc)
 {
   unsigned int nbTv = 0;
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   const Site* site = 0;
   while (si->hasMoreSites())
   {
@@ -498,14 +524,13 @@ unsigned int SequenceStatistics::numberOfTransversions(const PolymorphismSequenc
     // if (SiteTools::isConstant(*site) || SiteTools::isTriplet(*site)) continue;
     if (SiteTools::getNumberOfDistinctCharacters(*site) != 2)
       continue;
-    vector<int> seq = site->getContent();
-    int state1 = seq[0];
-    int state2 = seq[0];
-    for (size_t i = 1; i < seq.size(); i++)
+    int state1 = (*site)[0];
+    int state2 = (*site)[0];
+    for (size_t i = 1; i < site->size(); i++)
     {
-      if (state1 != seq[i])
+      if (state1 != (*site)[i])
       {
-        state2 = seq[i];
+        state2 = (*site)[i];
         break;
       }
     }
@@ -518,12 +543,12 @@ unsigned int SequenceStatistics::numberOfTransversions(const PolymorphismSequenc
   return nbTv;
 }
 
-double SequenceStatistics::ratioOfTransitionsTransversions(const PolymorphismSequenceContainer& psc) throw (Exception)
+double SequenceStatistics::ratioOfTransitionsTransversions(const PolymorphismSequenceContainer& psc)
 {
   // return (double) getNumberOfTransitions(psc)/getNumberOfTransversions(psc);
   double nbTs = 0;
   double nbTv = 0;
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   const Site* site = 0;
   vector<int> state(2);
   while (si->hasMoreSites())
@@ -567,7 +592,7 @@ unsigned int SequenceStatistics::numberOfSitesWithStopCodon(const PolymorphismSe
   if (!AlphabetTools::isCodonAlphabet(psc.getAlphabet()))
     throw AlphabetMismatchException("SequenceStatistics::stopCodonSiteNumber(). PolymorphismSequenceContainer must be with a codon alphabet.", psc.getAlphabet());
 
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (gapflag)
     si.reset(new NoGapSiteContainerIterator(psc));
   else
@@ -585,7 +610,7 @@ unsigned int SequenceStatistics::numberOfSitesWithStopCodon(const PolymorphismSe
 
 unsigned int SequenceStatistics::numberOfMonoSitePolymorphicCodons(const PolymorphismSequenceContainer& psc, bool stopflag, bool gapflag)
 {
-  auto_ptr<ConstSiteIterator> si;
+  unique_ptr<ConstSiteIterator> si;
   if (stopflag)
     si.reset(new CompleteSiteContainerIterator(psc));
   else
@@ -608,7 +633,7 @@ unsigned int SequenceStatistics::numberOfMonoSitePolymorphicCodons(const Polymor
 
 unsigned int SequenceStatistics::numberOfSynonymousPolymorphicCodons(const PolymorphismSequenceContainer& psc, const GeneticCode& gc)
 {
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   unsigned int s = 0;
   const Site* site;
   while (si->hasMoreSites())
@@ -701,7 +726,7 @@ double SequenceStatistics::meanNumberOfNonSynonymousSites(const PolymorphismSequ
 unsigned int SequenceStatistics::numberOfSynonymousSubstitutions(const PolymorphismSequenceContainer& psc, const GeneticCode& gc, double freqmin)
 {
   size_t st = 0, sns = 0;
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   const Site* site = 0;
   while (si->hasMoreSites())
   {
@@ -715,7 +740,7 @@ unsigned int SequenceStatistics::numberOfSynonymousSubstitutions(const Polymorph
 unsigned int SequenceStatistics::numberOfNonSynonymousSubstitutions(const PolymorphismSequenceContainer& psc, const GeneticCode& gc, double freqmin)
 {
   unsigned int sns = 0;
-  auto_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
+  unique_ptr<ConstSiteIterator> si(new CompleteSiteContainerIterator(psc));
   const Site* site = 0;
   while (si->hasMoreSites())
   {
@@ -727,9 +752,9 @@ unsigned int SequenceStatistics::numberOfNonSynonymousSubstitutions(const Polymo
 
 vector<unsigned int> SequenceStatistics::fixedDifferences(const PolymorphismSequenceContainer& pscin, const PolymorphismSequenceContainer& pscout, PolymorphismSequenceContainer& psccons, const GeneticCode& gc)
 {
-  auto_ptr<ConstSiteIterator> siIn(new CompleteSiteContainerIterator(pscin));
-  auto_ptr<ConstSiteIterator> siOut(new CompleteSiteContainerIterator(pscout));
-  auto_ptr<ConstSiteIterator> siCons(new CompleteSiteContainerIterator(psccons));
+  unique_ptr<ConstSiteIterator> siIn(new CompleteSiteContainerIterator(pscin));
+  unique_ptr<ConstSiteIterator> siOut(new CompleteSiteContainerIterator(pscout));
+  unique_ptr<ConstSiteIterator> siCons(new CompleteSiteContainerIterator(psccons));
   const Site* siteIn = 0;
   const Site* siteOut = 0;
   const Site* siteCons = 0;
@@ -758,12 +783,12 @@ vector<unsigned int> SequenceStatistics::mkTable(const PolymorphismSequenceConta
     psctot.addSequence(outgroup.getSequence(i));
     psctot.setAsOutgroupMember(i + ingroup.getNumberOfSequences());
   }
-  auto_ptr<const PolymorphismSequenceContainer> psccomplet(PolymorphismSequenceContainerTools::getCompleteSites(psctot));
-  auto_ptr<const PolymorphismSequenceContainer> pscin     (PolymorphismSequenceContainerTools::extractIngroup(*psccomplet));
-  auto_ptr<const PolymorphismSequenceContainer> pscout    (PolymorphismSequenceContainerTools::extractOutgroup(*psccomplet));
-  auto_ptr<const Sequence> consensusIn (SiteContainerTools::getConsensus(*pscin, "consensusIn"));
-  auto_ptr<const Sequence> consensusOut(SiteContainerTools::getConsensus(*pscout, "consensusOut"));
-  auto_ptr<PolymorphismSequenceContainer> consensus(new PolymorphismSequenceContainer(ingroup.getAlphabet()));
+  unique_ptr<const PolymorphismSequenceContainer> psccomplet(PolymorphismSequenceContainerTools::getCompleteSites(psctot));
+  unique_ptr<const PolymorphismSequenceContainer> pscin     (PolymorphismSequenceContainerTools::extractIngroup(*psccomplet));
+  unique_ptr<const PolymorphismSequenceContainer> pscout    (PolymorphismSequenceContainerTools::extractOutgroup(*psccomplet));
+  unique_ptr<const Sequence> consensusIn (SiteContainerTools::getConsensus(*pscin, "consensusIn"));
+  unique_ptr<const Sequence> consensusOut(SiteContainerTools::getConsensus(*pscout, "consensusOut"));
+  unique_ptr<PolymorphismSequenceContainer> consensus(new PolymorphismSequenceContainer(ingroup.getAlphabet()));
   consensus->addSequence(*consensusIn);
   consensus->addSequence(*consensusOut);
   vector<unsigned int> u = SequenceStatistics::fixedDifferences(*pscin, *pscout, *consensus, gc);
@@ -788,51 +813,61 @@ double SequenceStatistics::neutralityIndex(const PolymorphismSequenceContainer&
 // Statistical tests
 // ******************************************************************************
 
-double SequenceStatistics::tajimaDss(const PolymorphismSequenceContainer& psc, bool gapflag) throw (ZeroDivisionException)
+double SequenceStatistics::tajimaDss(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown)
 {
-  unsigned int Sp = numberOfPolymorphicSites(psc, gapflag);
+  unsigned int Sp = numberOfPolymorphicSites(psc, gapflag, ignoreUnknown);
   if (Sp == 0)
     throw ZeroDivisionException("SequenceStatistics::tajimaDss. S should not be 0.");
   double S = static_cast<double>(Sp);
-  double tajima = tajima83(psc, gapflag);
-  double watterson = watterson75(psc, gapflag);
+  double tajima = tajima83(psc, gapflag, ignoreUnknown);
+  double watterson = watterson75(psc, gapflag, ignoreUnknown);
   size_t n = psc.getNumberOfSequences();
   map<string, double> values = getUsefulValues_(n);
   return (tajima - watterson) / sqrt((values["e1"] * S) + (values["e2"] * S * (S - 1)));
 }
 
-double SequenceStatistics::tajimaDtnm(const PolymorphismSequenceContainer& psc, bool gapflag) throw (ZeroDivisionException)
+double SequenceStatistics::tajimaDtnm(const PolymorphismSequenceContainer& psc, bool gapflag, bool ignoreUnknown)
 {
   unsigned int etaP = totalNumberOfMutations(psc, gapflag);
   if (etaP == 0)
     throw ZeroDivisionException("SequenceStatistics::tajimaDtnm. Eta should not be 0.");
   double eta = static_cast<double>(etaP);
-  double tajima = tajima83(psc, gapflag);
+  double tajima = tajima83(psc, gapflag, ignoreUnknown);
   size_t n = psc.getNumberOfSequences();
   map<string, double> values = getUsefulValues_(n);
   double eta_a1 = eta / values["a1"];
   return (tajima - eta_a1) / sqrt((values["e1"] * eta) + (values["e2"] * eta * (eta - 1)));
 }
 
-double SequenceStatistics::fuLiD(const PolymorphismSequenceContainer& ingroup, const PolymorphismSequenceContainer& outgroup, bool original) throw (ZeroDivisionException)
+double SequenceStatistics::fuLiD(
+    const PolymorphismSequenceContainer& ingroup,
+    const PolymorphismSequenceContainer& outgroup,
+    bool useNbSingletons,
+    bool useNbSegregatingSites)
 {
   size_t n = ingroup.getNumberOfSequences();
   map<string, double> values = getUsefulValues_(n);
   double vD = getVD_(n, values["a1"], values["a2"], values["cn"]);
   double uD = getUD_(values["a1"], vD);
-  unsigned int etaP = totalNumberOfMutations(ingroup);
+  unsigned int etaP = 0;
+  if (useNbSegregatingSites)
+    etaP = numberOfPolymorphicSites(ingroup);
+  else
+    etaP = totalNumberOfMutations(ingroup);
   if (etaP == 0)
     throw ZeroDivisionException("SequenceStatistics::fuLiD. Eta should not be 0.");
   double eta = static_cast<double>(etaP);
   double etae = 0.;
-  if (original)
+  if (useNbSingletons)
     etae = static_cast<double>(numberOfSingletons(outgroup));
   else
     etae = static_cast<double>(totalNumberOfMutationsOnExternalBranches(ingroup, outgroup));  // added by Khalid 13/07/2005
   return (eta - (values["a1"] * etae)) / sqrt((uD * eta) + (vD * eta * eta));
 }
 
-double SequenceStatistics::fuLiDStar(const PolymorphismSequenceContainer& group) throw (ZeroDivisionException)
+double SequenceStatistics::fuLiDStar(
+    const PolymorphismSequenceContainer& group,
+    bool useNbSegregatingSites)
 {
   size_t n = group.getNumberOfSequences();
   double nn = static_cast<double>(n);
@@ -840,11 +875,16 @@ double SequenceStatistics::fuLiDStar(const PolymorphismSequenceContainer& group)
   map<string, double> values = getUsefulValues_(n);
   double vDs = getVDstar_(n, values["a1"], values["a2"], values["dn"]);
   double uDs = getUDstar_(n, values["a1"], vDs);
-  double eta = static_cast<double>(totalNumberOfMutations(group));
-  if (eta == 0.)
+  unsigned int etaP = 0;  
+  if (useNbSegregatingSites)
+    etaP = numberOfPolymorphicSites(group);
+  else 
+    etaP = totalNumberOfMutations(group);
+  if (etaP == 0)
     throw ZeroDivisionException("eta should not be null");
+  double eta = static_cast<double>(etaP);
   double etas = static_cast<double>(numberOfSingletons(group));
-
+ 
   // Fu & Li 1993
   return ((_n * eta) - (values["a1"] * etas)) / sqrt(uDs * eta + vDs * eta * eta);
 
@@ -854,7 +894,11 @@ double SequenceStatistics::fuLiDStar(const PolymorphismSequenceContainer& group)
    */
 }
 
-double SequenceStatistics::fuLiF(const PolymorphismSequenceContainer& ingroup, const PolymorphismSequenceContainer& outgroup, bool original) throw (ZeroDivisionException)
+double SequenceStatistics::fuLiF(
+    const PolymorphismSequenceContainer& ingroup,
+    const PolymorphismSequenceContainer& outgroup,
+    bool useNbSingletons,
+    bool useNbSegregatingSites)
 {
   size_t n = ingroup.getNumberOfSequences();
   double nn = static_cast<double>(n);
@@ -862,18 +906,25 @@ double SequenceStatistics::fuLiF(const PolymorphismSequenceContainer& ingroup, c
   double pi = tajima83(ingroup, true);
   double vF = (values["cn"] + values["b2"] - 2. / (nn - 1.)) / (pow(values["a1"], 2) + values["a2"]);
   double uF = ((1. + values["b1"] - (4. * ((nn + 1.) / ((nn - 1.) * (nn - 1.)))) * (values["a1n"] - (2. * nn) / (nn + 1.))) / values["a1"]) - vF;
-  double eta = static_cast<double>(totalNumberOfMutations(ingroup));
-  if (eta == 0.)
+  unsigned int etaP = 0;  
+  if (useNbSegregatingSites)
+    etaP = numberOfPolymorphicSites(ingroup);
+  else 
+    etaP = totalNumberOfMutations(ingroup);
+  if (etaP == 0)
     throw ZeroDivisionException("eta should not be null");
+  double eta = static_cast<double>(etaP);
   double etae = 0.;
-  if (original)
+  if (useNbSingletons)
     etae = static_cast<double>(numberOfSingletons(outgroup));
   else
     etae = static_cast<double>(totalNumberOfMutationsOnExternalBranches(ingroup, outgroup));  // added by Khalid 13/07/2005
   return (pi - etae) / sqrt(uF * eta + vF * eta * eta);
 }
 
-double SequenceStatistics::fuLiFStar(const PolymorphismSequenceContainer& group) throw (ZeroDivisionException)
+double SequenceStatistics::fuLiFStar(
+    const PolymorphismSequenceContainer& group,
+    bool useNbSegregatingSites)
 {
   double n = static_cast<double>(group.getNumberOfSequences());
   map<string, double> values = getUsefulValues_(group.getNumberOfSequences());
@@ -886,9 +937,14 @@ double SequenceStatistics::fuLiFStar(const PolymorphismSequenceContainer& group)
   // Simonsen et al. 1995
   double vFs = (((2 * n * n * n + 110 * n * n - 255 * n + 153) / (9 * n * n * (n - 1))) + ((2 * (n - 1) * values["a1"]) / (n * n)) - 8 * values["a2"] / n) / (pow(values["a1"], 2) + values["a2"]);
   double uFs = (((4 * n * n + 19 * n + 3 - 12 * (n + 1) * values["a1n"]) / (3 * n * (n - 1))) / values["a1"]) - vFs;
-  double eta = static_cast<double>(totalNumberOfMutations(group));
-  if (eta == 0.)
+  unsigned int etaP = 0;  
+  if (useNbSegregatingSites)
+    etaP = numberOfPolymorphicSites(group);
+  else 
+    etaP = totalNumberOfMutations(group);
+  if (etaP == 0)
     throw ZeroDivisionException("eta should not be null");
+  double eta = static_cast<double>(etaP);
   double etas = static_cast<double>(numberOfSingletons(group));
   // Fu & Li 1993
   // Simonsen et al. 1995
@@ -1008,7 +1064,7 @@ PolymorphismSequenceContainer* SequenceStatistics::generateLdContainer(const Pol
 /* Pairwise LD and distance measures */
 /*************************************/
 
-Vdouble SequenceStatistics::pairwiseDistances1(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::pairwiseDistances1(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   // get Positions with sites of interest
   SiteSelection ss;
@@ -1064,7 +1120,7 @@ Vdouble SequenceStatistics::pairwiseDistances1(const PolymorphismSequenceContain
   return dist;
 }
 
-Vdouble SequenceStatistics::pairwiseDistances2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::pairwiseDistances2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   SiteSelection ss;
   for (size_t i = 0; i < psc.getNumberOfSites(); i++)
@@ -1142,7 +1198,7 @@ Vdouble SequenceStatistics::pairwiseDistances2(const PolymorphismSequenceContain
   return distance;
 }
 
-Vdouble SequenceStatistics::pairwiseD(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::pairwiseD(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   PolymorphismSequenceContainer* newpsc = SequenceStatistics::generateLdContainer(psc, keepsingleton, freqmin);
   Vdouble D;
@@ -1175,7 +1231,7 @@ Vdouble SequenceStatistics::pairwiseD(const PolymorphismSequenceContainer& psc,
   return D;
 }
 
-Vdouble SequenceStatistics::pairwiseDprime(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::pairwiseDprime(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   PolymorphismSequenceContainer* newpsc = SequenceStatistics::generateLdContainer(psc, keepsingleton, freqmin);
   Vdouble Dprime;
@@ -1231,7 +1287,7 @@ Vdouble SequenceStatistics::pairwiseDprime(const PolymorphismSequenceContainer&
   return Dprime;
 }
 
-Vdouble SequenceStatistics::pairwiseR2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::pairwiseR2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   PolymorphismSequenceContainer* newpsc = SequenceStatistics::generateLdContainer(psc, keepsingleton, freqmin);
   Vdouble R2;
@@ -1269,203 +1325,126 @@ Vdouble SequenceStatistics::pairwiseR2(const PolymorphismSequenceContainer& psc,
 /* Global LD and distance measures */
 /***********************************/
 
-double SequenceStatistics::meanD(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::meanD(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
   Vdouble D = pairwiseD(psc, keepsingleton, freqmin);
   return VectorTools::mean<double, double>(D);
 }
 
-double SequenceStatistics::meanDprime(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::meanDprime(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin);
-    return VectorTools::mean<double, double>(Dprime);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin);
+  return VectorTools::mean<double, double>(Dprime);
 }
 
-double SequenceStatistics::meanR2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::meanR2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble R2 = SequenceStatistics::pairwiseR2(psc, keepsingleton, freqmin);
-    return VectorTools::mean<double, double>(R2);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble R2 = SequenceStatistics::pairwiseR2(psc, keepsingleton, freqmin);
+  return VectorTools::mean<double, double>(R2);
 }
 
-double SequenceStatistics::meanDistance1(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::meanDistance1(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble dist = pairwiseDistances1(psc, keepsingleton, freqmin);
-    return VectorTools::mean<double, double>(dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble dist = pairwiseDistances1(psc, keepsingleton, freqmin);
+  return VectorTools::mean<double, double>(dist);
 }
 
-double SequenceStatistics::meanDistance2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::meanDistance2(const PolymorphismSequenceContainer& psc, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble dist = pairwiseDistances2(psc, keepsingleton, freqmin);
-    return VectorTools::mean<double, double>(dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble dist = pairwiseDistances2(psc, keepsingleton, freqmin);
+  return VectorTools::mean<double, double>(dist);
 }
 
 /**********************/
 /* Regression methods */
 /**********************/
 
-double SequenceStatistics::originRegressionD(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::originRegressionD(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble D = pairwiseD(psc, keepsingleton, freqmin) - 1;
-    Vdouble dist;
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    return VectorTools::sum(D * dist) / VectorTools::sum(dist * dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble D = pairwiseD(psc, keepsingleton, freqmin) - 1;
+  Vdouble dist;
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  return VectorTools::sum(D * dist) / VectorTools::sum(dist * dist);
 }
 
-double SequenceStatistics::originRegressionDprime(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::originRegressionDprime(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin) - 1;
-    Vdouble dist;
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    return VectorTools::sum(Dprime * dist) / VectorTools::sum(dist * dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin) - 1;
+  Vdouble dist;
+  if (distance1)
+   dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  return VectorTools::sum(Dprime * dist) / VectorTools::sum(dist * dist);
 }
 
-double SequenceStatistics::originRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::originRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin) - 1;
-    Vdouble dist;
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    return VectorTools::sum(R2 * dist) / VectorTools::sum(dist * dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin) - 1;
+  Vdouble dist;
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  return VectorTools::sum(R2 * dist) / VectorTools::sum(dist * dist);
 }
 
-Vdouble SequenceStatistics::linearRegressionD(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::linearRegressionD(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble D = pairwiseD(psc, keepsingleton, freqmin);
-    Vdouble dist;
-    Vdouble reg(2);
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    reg[0] = VectorTools::cov<double, double>(dist, D) / VectorTools::var<double, double>(dist);
-    reg[1] = VectorTools::mean<double, double>(D) - reg[0] * VectorTools::mean<double, double>(dist);
-    return reg;
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble D = pairwiseD(psc, keepsingleton, freqmin);
+  Vdouble dist;
+  Vdouble reg(2);
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  reg[0] = VectorTools::cov<double, double>(dist, D) / VectorTools::var<double, double>(dist);
+  reg[1] = VectorTools::mean<double, double>(D) - reg[0] * VectorTools::mean<double, double>(dist);
+  return reg;
 }
 
-Vdouble SequenceStatistics::linearRegressionDprime(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::linearRegressionDprime(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin);
-    Vdouble dist;
-    Vdouble reg(2);
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    reg[0] = VectorTools::cov<double, double>(dist, Dprime) / VectorTools::var<double, double>(dist);
-    reg[1] = VectorTools::mean<double, double>(Dprime) - reg[0] * VectorTools::mean<double, double>(dist);
-    return reg;
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble Dprime = pairwiseDprime(psc, keepsingleton, freqmin);
+  Vdouble dist;
+  Vdouble reg(2);
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  reg[0] = VectorTools::cov<double, double>(dist, Dprime) / VectorTools::var<double, double>(dist);
+  reg[1] = VectorTools::mean<double, double>(Dprime) - reg[0] * VectorTools::mean<double, double>(dist);
+  return reg;
 }
 
-Vdouble SequenceStatistics::linearRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+Vdouble SequenceStatistics::linearRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin);
-    Vdouble dist;
-    Vdouble reg(2);
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    reg[0] = VectorTools::cov<double, double>(dist, R2) / VectorTools::var<double, double>(dist);
-    reg[1] = VectorTools::mean<double, double>(R2) - reg[0] * VectorTools::mean<double, double>(dist);
-    return reg;
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin);
+  Vdouble dist;
+  Vdouble reg(2);
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  reg[0] = VectorTools::cov<double, double>(dist, R2) / VectorTools::var<double, double>(dist);
+  reg[1] = VectorTools::mean<double, double>(R2) - reg[0] * VectorTools::mean<double, double>(dist);
+  return reg;
 }
 
-double SequenceStatistics::inverseRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin) throw (DimensionException)
+double SequenceStatistics::inverseRegressionR2(const PolymorphismSequenceContainer& psc, bool distance1, bool keepsingleton, double freqmin)
 {
-  try
-  {
-    Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin);
-    Vdouble unit(R2.size(), 1);
-    Vdouble R2transformed = unit / R2 - 1;
-    Vdouble dist;
-    if (distance1)
-      dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
-    else
-      dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
-    return VectorTools::sum(R2transformed * dist) / VectorTools::sum(dist * dist);
-  }
-  catch (DimensionException& e)
-  {
-    throw e;
-  }
+  Vdouble R2 = pairwiseR2(psc, keepsingleton, freqmin);
+  Vdouble unit(R2.size(), 1);
+  Vdouble R2transformed = unit / R2 - 1;
+  Vdouble dist;
+  if (distance1)
+    dist = pairwiseDistances1(psc, keepsingleton, freqmin) / 1000;
+  else
+    dist = pairwiseDistances2(psc, keepsingleton, freqmin) / 1000;
+  return VectorTools::sum(R2transformed * dist) / VectorTools::sum(dist * dist);
 }
 
 /**********************/
@@ -1532,6 +1511,7 @@ void SequenceStatistics::testUsefulValues(std::ostream& s, size_t n)
 
 unsigned int SequenceStatistics::getNumberOfMutations_(const Site& site)
 {
+  //jdutheil 27/06/15: does not work if gaps and unknown!!!
   unsigned int tmp_count = 0;
   map<int, size_t> states_count;
   SymbolListTools::getCounts(site, states_count);
